<?php

namespace Tests\Feature;

use App\Models\User;
use App\Models\Retailer;
use App\Models\Product;
use Hash;
use Laravel\Sanctum\Sanctum;
use Tests\TestCase;

class ChangeActionTest extends TestCase
{
    public function test_success_change_value_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer1 = Retailer::factory()->create();
        $retailer2 = Retailer::factory()->create();
        $product1 = Product::factory()->create([
            'sku' => '999999',
            'price' => '1',
            'retailer_id' => $retailer1->id
        ]);
        $product2 = Product::factory()->create([
            'sku' => '999999',
            'price' => '2',
            'retailer_id' => $retailer2->id
        ]);
        $this->get('/nova/resources/products/lens/compare-product?field=price&retailer=' . $retailer1->id);
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=change-value&pivotAction=false&filters=W10%3D',[
            'resources' => $product2->id
        ]);
        $user->delete();
        $retailer1->delete();
        $retailer2->delete();
        $product1->delete();
        $product2->delete();
        $response->assertStatus(200);
    }

    public function test_change_value_without_resource_by_admin()
    {
        $user = User::factory()->create([
            'role' => 'admin',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer1 = Retailer::factory()->create();
        $product1 = Product::factory()->create([
            'sku' => '999999',
            'price' => '1',
            'retailer_id' => $retailer1->id
        ]);
        $this->get('/nova/resources/products/lens/compare-product?field=price&retailer=' . $retailer1->id);
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=change-value&pivotAction=false&filters=W10%3D',[
            
        ]);
        $user->delete();
        $retailer1->delete();
        $product1->delete();
        $response->assertStatus(403);
    }

    public function test_success_change_value_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer1 = Retailer::factory()->create();
        $retailer2 = Retailer::factory()->create();
        $product1 = Product::factory()->create([
            'sku' => '999999',
            'price' => '1',
            'retailer_id' => $retailer1->id
        ]);
        $product2 = Product::factory()->create([
            'sku' => '999999',
            'price' => '2',
            'retailer_id' => $retailer2->id
        ]);
        $this->get('/nova/resources/products/lens/compare-product?field=price&retailer=' . $retailer1->id);
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=change-value&pivotAction=false&filters=W10%3D',[
            'resources' => $product2->id
        ]);
        $user->delete();
        $retailer1->delete();
        $retailer2->delete();
        $product1->delete();
        $product2->delete();
        $response->assertStatus(200);
    }

    public function test_change_value_without_resource_by_analyst()
    {
        $user = User::factory()->create([
            'role' => 'analyst',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer1 = Retailer::factory()->create();
        $product1 = Product::factory()->create([
            'sku' => '999999',
            'price' => '1',
            'retailer_id' => $retailer1->id
        ]);
        $this->get('/nova/resources/products/lens/compare-product?field=price&retailer=' . $retailer1->id);
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=change-value&pivotAction=false&filters=W10%3D',[
            
        ]);
        $user->delete();
        $retailer1->delete();
        $product1->delete();
        $response->assertStatus(403);
    }

    public function test_change_value_by_common()
    {
        $user = User::factory()->create([
            'role' => 'common',
            'password' => Hash::make('123123123'),
        ]);
        Sanctum::actingAs(
            $user,
            ['*']
        );
        $retailer1 = Retailer::factory()->create();
        $retailer2 = Retailer::factory()->create();
        $product1 = Product::factory()->create([
            'sku' => '999999',
            'price' => '1',
            'retailer_id' => $retailer1->id
        ]);
        $product2 = Product::factory()->create([
            'sku' => '999999',
            'price' => '2',
            'retailer_id' => $retailer2->id
        ]);
        $this->get('/nova/resources/products/lens/compare-product?field=price&retailer=' . $retailer1->id);
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=change-value&pivotAction=false&filters=W10%3D',[
            'resources' => $product2->id
        ]);
        $user->delete();
        $retailer1->delete();
        $retailer2->delete();
        $product1->delete();
        $product2->delete();
        $response->assertStatus(403);
    }

    public function test_change_value_by_unauthorized()
    {
        $retailer1 = Retailer::factory()->create();
        $retailer2 = Retailer::factory()->create();
        $product1 = Product::factory()->create([
            'sku' => '999999',
            'price' => '1',
            'retailer_id' => $retailer1->id
        ]);
        $product2 = Product::factory()->create([
            'sku' => '999999',
            'price' => '2',
            'retailer_id' => $retailer2->id
        ]);
        $this->get('/nova/resources/products/lens/compare-product?field=price&retailer=' . $retailer1->id);
        $response = $this->post('/nova-api/products/lens/compare-product/action?action=change-value&pivotAction=false&filters=W10%3D',[
            'resources' => $product2->id
        ]);
        $retailer1->delete();
        $retailer2->delete();
        $product1->delete();
        $product2->delete();
        $response->assertStatus(401);
    }
}
